require 'csv'
require 'ostruct'
require 'faraday'
require 'faraday_middleware'
require 'typhoeus'
require 'typhoeus/adapters/faraday'

module Ratr
  class Application
    attr_reader :file
    def initialize(file = "examples/movies_small.csv")
      @file = file
    end

    def call
      # read incoming csv file
      movies = {}
      movie_key = 0
      CSV.foreach(file) do |row|
        #assign each movie a title and year. Note: year is not guaranteed
        movies[movie_key] = OpenStruct.new(title: row[0], year: row[1])
        movie_key += 1
      end

      manager = Ratr::PoolManager.new(sources)
      manager.process(movies)
      manager.join

      movies.each do |index, movie|
        scores = manager.merged_results[index].compact

        movie.average = Ratr::AverageCalculator.call(scores)
      end

      # sort the movies collection by average rating -- then display.
      output = []
      movies.values.sort {|a,b| b.average <=> a.average}.each do |movie|
        output << "#{movie.title} (#{movie.year}): #{movie.average}"
      end
      
      return output
    end

    private

    def sources
      @sources ||= [omdb, rotten_tomatoes]
    end

    def omdb
      @omdb ||= Ratr::Source.new(Ratr::ParallelHttpProcessor, Ratr::OmdbHttpWrapper.new)
    end

    def rotten_tomatoes
      @rotten_tomatoes = Ratr::Source.new(Ratr::SerialHttpProcessor, Ratr::RottenTomatoesHttpWrapper.new)
    end
  end
end
