module Ratr
  class RottenTomatoesHttpWrapper
    INVALID_VALUES = [-1]
    SCALAR = 10.0

    attr_reader :url, :apikey
    def initialize(options = {})
      @apikey = options.fetch(:apikey, "ww8qgxbhjbqudvupbr8sqd7x")
      @url = options.fetch(:url, 'http://api.rottentomatoes.com')
    end

    # RT is really the problem child when it comes to requesting
    # multiple searches at once. It will return a 403 (forbidden)
    # code on rate limit exceeded which is a pain in the bum.
    #
    # Faraday has ERR::Timeout retries built in, but this is
    # unfortunatley not able to tap into it. The best I can do
    # is to roll my own custom retry mechanism. The idea is
    # to try and retry sooner than the 0.02 under most circumstances.
    def get(movie)
      retry_count = 3
      begin
        adapter.get do |request|
          request.url '/api/public/v1.0/movies.json'
          request.params['q'] = "#{movie.title} #{movie.year}"
          request.params['apikey'] = apikey
          request.params['page_limit'] = 1
        end
      rescue Faraday::Error::ClientError
        case retry_count
        when 3
          sleep 0.05
        when 2
          sleep 0.1
        when 1
          sleep 0.2
        end
        retry_count -=1

        retry if retry_count > 0
        OpenStruct.new(body: {"movies" => []})
      end
    end

    def scrub(responses)
      responses.reduce({}) do |memo, (index, response)|
        movie = response.body["movies"][0]
        memo[index] = []

        if movie
          ratings  = movie["ratings"]
          critic   = ratings["critics_score"]
          audience = ratings["audience_score"]

          memo[index].push(audience.to_f / SCALAR) if audience && !INVALID_VALUES.include?(audience)
          memo[index].push(critic.to_f / SCALAR) if critic && !INVALID_VALUES.include?(critic)
        end

        memo
      end
    end

    def adapter
      @adapter ||= Faraday.new(url: url) do |faraday|
        faraday.request :json
        faraday.response :json
        faraday.response :raise_error
        faraday.adapter  :typhoeus
      end
    end
  end
end
